@@ -5,11 +5,13 @@ import glob
 pyFiles = glob.glob("lib/doit/*.py") + glob.glob("tests/*.py")
 
 def task_checker():
+    """run pyflakes on all project files"""
     for file in pyFiles:
-        yield {'action': "pyflakes %s"% file,
+        yield {'actions': ["pyflakes %s"% file],
                'name':file,
                'dependencies':(file,)}
 
 def task_nose():
-    return {'action':"nosetests",
+    """run unit-tests"""
+    return {'actions':["nosetests"],
             'dependencies':pyFiles}
