@@ -4,7 +4,7 @@ import sys
 import inspect
 
 from doit.util import isgenerator
-from doit.task import InvalidTask, BaseTask, GroupTask, dict_to_task
+from doit.task import InvalidTask, Task, dict_to_task
 from doit import dependency
 from doit import runner
 
@@ -31,14 +31,14 @@ def get_module(dodoFile):
     sys.path.insert(0, base_path)
     # file specified on dodo file are relative to itself.
     os.chdir(base_path)
-    # get module containing the tasks
-    try:
-        return __import__(os.path.splitext(file_name)[0])
-    except ImportError:
-        msg = ("Could not find dodo file '%s'. " +
-               "Please use '-f' to specify file name. " +
-               "Use the command 'dodo-sample' to view a sample.")
+    if not os.path.exists(file_name):
+        msg = ("Could not find dodo file '%s'.\n" +
+               "Please use '-f' to specify file name.\n" +
+               "Use the command 'dodo-sample' to view a sample dodo file.\n")
         raise InvalidDodoFile(msg % dodoFile)
+    # get module containing the tasks
+    return __import__(os.path.splitext(file_name)[0])
+
 
 def load_task_generators(dodo_module, command_names=()):
     """Loads a python file and extracts its task generator functions.
@@ -48,7 +48,7 @@ def load_task_generators(dodo_module, command_names=()):
     @param dodo_module: (module) module containing the tasks
     @param command_names: (list - str) blacklist for task names
     @return (dict):
-     - task_list (list) of BaseTasks in the order they were defined on the file
+     - task_list (list) of Tasks in the order they were defined on the file
      - default_tasks (list) of tasks to be executed by default
     """
 
@@ -61,6 +61,7 @@ def load_task_generators(dodo_module, command_names=()):
         # ignore functions that are not a task (by its name)
         if not name.startswith(TASK_STRING):
             continue
+        # remove TASK_STRING prefix from name
         task_name = name[prefix_len:]
         # tasks cant have name of commands
         if task_name in command_names:
@@ -70,7 +71,6 @@ def load_task_generators(dodo_module, command_names=()):
         # get line number where function is defined
         line = inspect.getsourcelines(ref)[1]
         # add to list task generator functions
-        # remove TASK_STRING prefix from name
         funcs.append((task_name, ref, line))
 
     # sort by the order functions were defined (line number)
@@ -79,7 +79,7 @@ def load_task_generators(dodo_module, command_names=()):
     # generate all tasks
     task_list = []
     for name, ref, line in funcs:
-        task_list.extend(generate_tasks(name, ref()))
+        task_list.extend(generate_tasks(name, ref(), ref.__doc__))
 
     # get default tasks
     default_tasks = getattr(dodo_module, 'DEFAULT_TASKS', None)
@@ -91,11 +91,12 @@ def load_task_generators(dodo_module, command_names=()):
     return {'task_list': task_list,
             'default_tasks': default_tasks}
 
-def generate_tasks(name, gen_result):
+def generate_tasks(name, gen_result, gen_doc=None):
     """Create tasks from a task generator result.
 
     @param name: (string) name of taskgen function
     @param gen_result: value returned by a task generator function
+    @param gen_doc: (string/None) docstring from the task generator function
     @return: (tuple) task,list of subtasks
     """
     # task described as a dictionary
@@ -104,11 +105,17 @@ def generate_tasks(name, gen_result):
             raise InvalidTask("Task %s. Only subtasks use field name."%name)
 
         gen_result['name'] = name
+
+        # Use task generator docstring
+        # if no doc present in task dict
+        if not 'doc' in gen_result:
+            gen_result['doc'] = gen_doc
+
         return [dict_to_task(gen_result)]
 
     # a generator
     if isgenerator(gen_result):
-        group_task = GroupTask(name, None)
+        group_task = Task(name, None, doc=gen_doc)
         tasks = [group_task]
         # the generator return subtasks as dictionaries .
         for task_dict in gen_result:
@@ -129,8 +136,8 @@ def generate_tasks(name, gen_result):
         group_task.task_dep = [task.name for task in tasks[1:]]
         return tasks
 
-    # if not a dictionary nor a generator. "task" is the action itself.
-    return [dict_to_task({'name':name,'action':gen_result})]
+    raise InvalidTask("Task %s. Must return a dictionary. got %s" %
+                      (name, type(gen_result)))
 
 
 # this name is confusing with task.setup which it doesnt have any relation...
@@ -142,9 +149,9 @@ class TaskSetup(object):
 
     @ivar filter: (sequence of strings) selection of tasks to execute
     @ivar tasks: (dict) Key: task name ([taskgen.]name)
-                               Value: L{BaseTask} instance
+                               Value: L{Task} instance
     @ivar targets: (dict) Key: fileName
-                          Value: L{BaseTask} instance
+                          Value: L{Task} instance
     """
 
     def __init__(self, task_list, filter_=None):
@@ -160,9 +167,9 @@ class TaskSetup(object):
 
         # sanity check and create tasks dict
         for task in task_list:
-            # task must be a BaseTask
-            if not isinstance(task, BaseTask):
-                msg = "Task must an instance of BaseTask class. %s"
+            # task must be a Task
+            if not isinstance(task, Task):
+                msg = "Task must an instance of Task class. %s"
                 raise InvalidTask(msg % (task.__class__))
             # task name must be unique
             if task.name in self.tasks:
@@ -216,7 +223,7 @@ class TaskSetup(object):
             elif filter_ in self.targets:
                 selectedTask.append(self.targets[filter_].name)
             else:
-                msg = ('"%s" must be a sub-commandm, a task, or a target.\n' +
+                msg = ('"%s" must be a sub-command, a task, or a target.\n' +
                        'Type "doit help" to see available sub-commands.\n' +
                        'Type "doit list" to see available tasks')
                 raise InvalidCommand(msg % filter_)
@@ -277,24 +284,39 @@ def doit_run(dependencyFile, task_list, filter_=None,
                             verbosity, alwaysExecute)
 
 
+def doit_clean(task_list, clean_tasks):
+    """Clean tasks
+    @param task_list (list - L{Task}): list of all tasks from dodo file
+    @param clean_tasks (list - string): tasks bo be clean. clean all if
+                                        empty list.
+    """
+    if not clean_tasks:
+        for task_ in task_list:
+            task_.clean()
+    else:
+        tasks = dict([(t.name, t) for t in task_list])
+        for name in clean_tasks:
+            tasks[name].clean()
 
-def doit_list(task_list, printSubtasks):
+
+def doit_list(task_list, printSubtasks, quiet=False):
     """List task generators, in the order they were defined.
 
     @param printSubtasks: (bool) print subtasks
     """
-    print "==== Tasks ===="
     for task in task_list:
         if (not task.is_subtask) or printSubtasks:
-            print task.name
-    print "="*25,"\n"
+            task_str = task.name
+            if not quiet and task.doc:
+                task_str += " : %s" % task.doc
+            print task_str
     return 0
 
 
 def doit_forget(dbFileName, taskList, forgetTasks):
     """remove saved data successful runs from DB
     @param dbFileName: (str)
-    @param task_list: (BaseTask) tasks from dodo file
+    @param task_list: (Task) tasks from dodo file
     @param forget_tasks: (list - str) tasks to be removed. remove all if
                          empty list.
     """
@@ -315,7 +337,7 @@ def doit_forget(dbFileName, taskList, forgetTasks):
             group = [taskName]
             while group:
                 to_forget = group.pop(0)
-                if tasks[to_forget].action is None:
+                if not tasks[to_forget].actions:
                     # get task dependencies only from group-task
                     group.extend(tasks[to_forget].task_dep)
                 # forget it - remove from dependency file
